#!/usr/bin/env python
import logging
from pathlib import Path

from edc_constants.constants import IGNORE
from edc_test_utils import DefaultTestSettings, func_main

app_name = "edc_sites"
base_dir = Path(__file__).absolute().parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    template_dirs=[str(base_dir / app_name / "tests" / "templates")],
    BASE_DIR=base_dir,
    DEBUG=True,
    KEY_PATH=str(base_dir / app_name / "tests" / "etc"),
    ETC_DIR=str(base_dir / app_name / "tests" / "etc"),
    AUTO_CREATE_KEYS=False,
    APP_NAME=app_name,
    SITE_ID=10,
    EDC_SITES_MODULE_NAME="edc_sites.tests.sites",
    EDC_NAVBAR_VERIFY_ON_LOAD=IGNORE,
    SUBJECT_VISIT_MODEL="edc_visit_tracking.subjectvisit",
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields",
        "edc_auth.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_listboard.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_review_dashboard.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "multisite",
        "edc_sites",
    ],
    USE_I18N=True,
    USE_L10N=True,
    USE_TZ=True,
    LANGUAGE_CODE="en",
    use_test_urls=True,
    add_dashboard_middleware=True,
).settings


def main():
    tests = ["edc_sites"]
    func_main(project_settings, *tests)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from __future__ import annotations

from typing import TYPE_CHECKING

from .get_user_codenames_or_raise import get_user_codenames_or_raise

if TYPE_CHECKING:
    from django.contrib.auth.models import User


def get_change_codenames(user: User) -> list[str]:
    """Returns a list of codenames for this user that are prefixed
    with add/change/delete.
    """
    codenames = get_user_codenames_or_raise(user)
    return [c for c in codenames if "add" in c or "change" in c or "delete" in c]



from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

site_auths.add_post_update_func(
    "edc_sites", remove_default_model_permissions_from_edc_permissions
)

view_auditallsites_codename = "view_auditallsites"
site_auths.add_custom_permissions_tuples(
    model="edc_sites.edcpermissions",
    codename_tuples=[
        (f"edc_sites.{view_auditallsites_codename}", "View all sites as auditor")
    ],
)



from __future__ import annotations

from django.core.exceptions import ObjectDoesNotExist

from .auths import view_auditallsites_codename
from .models import SiteProfile
from .site import SiteNotRegistered, sites


class SiteViewMixin:
    def get_context_data(self, **kwargs) -> dict:
        try:
            site_profile = SiteProfile.objects.get(site__id=self.request.site.id)
        except ObjectDoesNotExist:
            site_profile = None
        kwargs.update(site_profile=site_profile)
        try:
            kwargs.update(site_title=site_profile.title)
        except AttributeError:
            if not sites.all():
                raise SiteNotRegistered(
                    "Unable to determine site profile 'title'. No sites have been registered! "
                )
            raise
        codename = f"edc_sites.{view_auditallsites_codename}"
        kwargs.update(has_view_auditallsites=self.request.user.has_perm(codename))
        return super().get_context_data(**kwargs)



import sys

from django.apps import AppConfig as DjangoAppConfig
from django.core.management.color import color_style
from django.db.models.signals import post_migrate

from .site import get_autodiscover_sites, sites

style = color_style()


def post_migrate_update_sites(sender=None, **kwargs):
    from edc_sites.utils import add_or_update_django_sites

    sys.stdout.write(style.MIGRATE_HEADING("Updating sites:\n"))

    for country in sites.countries:
        sys.stdout.write(style.MIGRATE_HEADING(f" (*) sites for {country} ...\n"))
        add_or_update_django_sites(verbose=True)
    sys.stdout.write("Done.\n")
    sys.stdout.flush()


class AppConfig(DjangoAppConfig):
    name = "edc_sites"
    verbose_name = "Edc Sites"
    has_exportable_data = True
    default_auto_field = "django.db.models.BigAutoField"
    include_in_administration_section = True

    def ready(self):
        if get_autodiscover_sites():
            post_migrate.connect(post_migrate_update_sites, sender=self)
            sys.stdout.write(f"Loading {self.verbose_name} ...\n")
            sites.autodiscover()



from __future__ import annotations

from typing import TYPE_CHECKING

from edc_auth.auth_objects import ACCOUNT_MANAGER_ROLE
from edc_auth.utils import get_codenames_for_role

if TYPE_CHECKING:
    from django.contrib.auth.models import User


def get_user_codenames_or_raise(user: User) -> list[str]:
    """Returns a list of all codenames for this user."""
    codenames = [
        perm.codename
        for perm in user.user_permissions.all()
        if perm.codename not in get_codenames_for_role(ACCOUNT_MANAGER_ROLE)
    ]
    if not codenames:
        raise PermissionError("User has not been allocated permission to anything.")
    return codenames
